@@ -80,15 +80,17 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
 
     let expanded = quote! {
         impl #crate_name::types::Type for #ident {
-            const NAME: #crate_name::types::TypeName = #crate_name::types::TypeName::Normal {
-                ty: #oai_typename,
-                format: ::std::option::Option::None,
-            };
             const IS_REQUIRED: bool = true;
 
-            type ValueType = Self;
+            type RawValueType = Self;
 
-            fn as_value(&self) -> ::std::option::Option<&Self> {
+            type RawElementValueType = Self;
+
+            fn name() -> ::std::borrow::Cow<'static, str> {
+                ::std::convert::Into::into(#oai_typename)
+            }
+
+            fn as_raw_value(&self) -> ::std::option::Option<&Self::RawValueType> {
                 ::std::option::Option::Some(self)
             }
 
@@ -97,17 +99,21 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
             }
 
             fn register(registry: &mut #crate_name::registry::Registry) {
-                registry.create_schema(#oai_typename, |registry| #crate_name::registry::MetaSchema {
+                registry.create_schema::<Self, _>(#oai_typename, |registry| #crate_name::registry::MetaSchema {
                     enum_items: ::std::vec![#(#enum_items),*],
-                    ..#crate_name::registry::MetaSchema::new(#oai_typename)
+                    ..#crate_name::registry::MetaSchema::new("string")
                 });
             }
+
+            fn raw_element_iter<'a>(&'a self) -> ::std::boxed::Box<dyn ::std::iter::Iterator<Item = &'a Self::RawElementValueType> + 'a> {
+                ::std::boxed::Box::new(::std::iter::IntoIterator::into_iter(self.as_raw_value()))
+            }
         }
 
         impl #crate_name::types::ParseFromJSON for #ident {
-            fn parse_from_json(value: #crate_name::serde_json::Value) -> #crate_name::types::ParseResult<Self> {
+            fn parse_from_json(value: #crate_name::__private::serde_json::Value) -> #crate_name::types::ParseResult<Self> {
                 match &value {
-                    #crate_name::serde_json::Value::String(item) => match item.as_str() {
+                    #crate_name::__private::serde_json::Value::String(item) => match item.as_str() {
                         #(#item_to_ident,)*
                         _ => ::std::result::Result::Err(#crate_name::types::ParseError::expected_type(value)),
                     }
@@ -117,34 +123,31 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
         }
 
         impl #crate_name::types::ParseFromParameter for #ident {
-            fn parse_from_parameter(value: ::std::option::Option<&str>) -> #crate_name::types::ParseResult<Self> {
+            fn parse_from_parameter(value: &str) -> #crate_name::types::ParseResult<Self> {
                 match value {
-                    ::std::option::Option::Some(value) => match value {
-                        #(#item_to_ident,)*
-                        _ => ::std::result::Result::Err(#crate_name::types::ParseError::custom("Expect a valid enumeration value.")),
-                    },
-                    _ => ::std::result::Result::Err(#crate_name::types::ParseError::expected_input()),
+                    #(#item_to_ident,)*
+                    _ => ::std::result::Result::Err(#crate_name::types::ParseError::custom("Expect a valid enumeration value.")),
                 }
             }
         }
 
         impl #crate_name::types::ToJSON for #ident {
-            fn to_json(&self) -> #crate_name::serde_json::Value {
+            fn to_json(&self) -> #crate_name::__private::serde_json::Value {
                 let name = match self {
                     #(#ident_to_item),*
                 };
-                #crate_name::serde_json::Value::String(::std::string::ToString::to_string(name))
+                #crate_name::__private::serde_json::Value::String(::std::string::ToString::to_string(name))
             }
         }
 
-        #[#crate_name::poem::async_trait]
+        #[#crate_name::__private::poem::async_trait]
         impl #crate_name::types::ParseFromMultipartField for #ident {
-            async fn parse_from_multipart(field: ::std::option::Option<#crate_name::poem::web::Field>) -> #crate_name::types::ParseResult<Self> {
+            async fn parse_from_multipart(field: ::std::option::Option<#crate_name::__private::poem::web::Field>) -> #crate_name::types::ParseResult<Self> {
                 use poem_openapi::types::ParseFromParameter;
                 match field {
                     ::std::option::Option::Some(field) => {
                         let s = field.text().await?;
-                        Self::parse_from_parameter(::std::option::Option::Some(&s))
+                        Self::parse_from_parameter(&s)
                     },
                     ::std::option::Option::None => ::std::result::Result::Err(#crate_name::types::ParseError::expected_input()),
                 }
